import os
from meteva.base.tool.path_tools import creat_path

def grib_to_nc1(grib_filename,output_dir = None):
    path_cd = grib_filename
    if (os.path.exists(path_cd)):
        # print(path_cd)
        filename, extension = os.path.splitext(path_cd)
        if output_dir is not None:
            output_dir = output_dir.replace("\\", "/")
            output_dir = output_dir + "/"
            creat_path(output_dir)
            dir, filename0 = os.path.split(filename)
            filename = output_dir + filename0
            filename = filename.replace("\\", "/")
        # print(filename)

        #path_out_grib_isobaricInhPa = filename + "_isobaric" + extension
        path_out_nc_isobaricInhPa = filename + ".nc"

        # start = time.time()
        #run_path = "grib_copy.exe -w typeOfLevel=surface,edition=1  " + path_cd + " " + path_out_grib_surface1
        #os.system(run_path)
        #run_path = "grib_copy.exe -w typeOfLevel=surface,edition=2  " + path_cd + " " + path_out_grib_surface2
        #os.system(run_path)
        #run_path = "grib_copy.exe -w typeOfLevel=isobaricInhPa  " + path_cd + " " + path_out_grib_isobaricInhPa
        #os.system(run_path)

        #os.system(r"grib_to_netcdf -o " + path_out_nc1_surface + " " + path_out_grib_surface1)
        #os.system(r"grib_to_netcdf -o " + path_out_nc2_surface + " " + path_out_grib_surface2)
        #os.system(r"grib_to_netcdf -o " + path_out_nc_isobaricInhPa + " " + path_out_grib_isobaricInhPa)
        os.system(r"grib_to_netcdf -o " + path_out_nc_isobaricInhPa+ " " + path_cd)
        # if (os.path.exists(path_cd)): os.remove(path_cd)
        result_list = []
        if (os.path.exists(path_out_nc_isobaricInhPa)): result_list.append(path_out_nc_isobaricInhPa)
        if len(result_list) > 1:
            print("转换生成如下文件：")
            for result in result_list:
                print(result)
            return result_list
        else:
            "未正常生成转换后的nc文件"
            return None

def grib_to_nc(grib_filename,output_dir = None):
    path_cd = grib_filename
    if (os.path.exists(path_cd)):
        #print(path_cd)
        filename,extension = os.path.splitext(path_cd)
        if output_dir is not None:
            output_dir = output_dir.replace("\\","/")
            output_dir = output_dir +"/"
            creat_path(output_dir)
            dir,filename0 = os.path.split(filename)
            filename = output_dir  + filename0
            filename = filename.replace("\\", "/")
        #print(filename)
        path_out_grib_surface1 = filename+"_surface1"+ extension
        path_out_grib_surface2 = filename + "_surface2" + extension
        path_out_grib_isobaricInhPa  = filename + "_isobaric" + extension
        path_out_nc1_surface = filename+"_surface1"+ ".nc"
        path_out_nc2_surface = filename+"_surface2"+ ".nc"
        path_out_nc_isobaricInhPa = filename + "_isobaric" + ".nc"

        # start = time.time()
        run_path = "grib_copy.exe -w typeOfLevel=surface,edition=1  " + path_cd + " " + path_out_grib_surface1
        os.system(run_path)
        run_path = "grib_copy.exe -w typeOfLevel=surface,edition=2  " + path_cd + " " + path_out_grib_surface2
        os.system(run_path)
        run_path = "grib_copy.exe -w typeOfLevel=isobaricInhPa  " + path_cd + " " + path_out_grib_isobaricInhPa
        os.system(run_path)

        os.system(r"grib_to_netcdf -o " + path_out_nc1_surface + " " + path_out_grib_surface1)
        os.system(r"grib_to_netcdf -o " + path_out_nc2_surface + " " + path_out_grib_surface2)
        os.system(r"grib_to_netcdf -o " + path_out_nc_isobaricInhPa + " " + path_out_grib_isobaricInhPa)

        #if (os.path.exists(path_cd)): os.remove(path_cd)
        if (os.path.exists(path_out_grib_isobaricInhPa)): os.remove(path_out_grib_isobaricInhPa)
        if (os.path.exists(path_out_grib_surface1)): os.remove(path_out_grib_surface1)
        if (os.path.exists(path_out_grib_surface2)): os.remove(path_out_grib_surface2)

        result_list = []
        if (os.path.exists(path_out_nc1_surface)): result_list.append(path_out_nc1_surface)
        if (os.path.exists(path_out_nc2_surface)): result_list.append(path_out_nc2_surface)
        if (os.path.exists(path_out_nc_isobaricInhPa)): result_list.append(path_out_nc_isobaricInhPa)
        if len(result_list)>1:
            print("转换生成如下文件：")
            for result in result_list:
                print(result)
            return result_list
        else:
            "未正常生成转换后的nc文件"
            return None





import pandas as pd
import numpy as np
import copy


def contingency_table_multicategory(ob, fo, grade_list=None,member_list=None,  save_path=None):
    '''
    multi_category_contingency_table 多分类预测列联表
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :param save_path: 保存地址
    :return: 内容为混淆矩阵的二维数组，shape为（类别数 + 1）×（类别数 + 1）
    '''
    conf_mx_list = []
    table_data_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    label = []
    if member_list is None:
        if new_Fo_shape[0] == 1:
            label.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)

    if grade_list is not None:
        index_list0 = ["<" + str(grade_list[0])]
        for index in range(len(grade_list) - 1):
            index_list0.append("[" + str(grade_list[index]) + "," + str(grade_list[index + 1]) + ")")
        index_list0.append(">=" + str(grade_list[-1]))
    else:
        new_fo = copy.deepcopy(fo).flatten()
        new_ob = copy.deepcopy(ob).flatten()
        fo_set = list(set(new_fo.tolist()))
        ob_set = list(set(new_ob.tolist()))
        fo_set.extend(ob_set)
        index_list0 = list(set(fo_set))
        index_list0.sort()
        if len(index_list0) > 30:
            print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
            return
    for line in range(new_Fo_shape[0]):
        fo_piece = new_Fo[line, :]
        if grade_list is not None:
            ngrade = len(grade_list) + 1
            conf_mx = np.zeros((ngrade, ngrade))
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):
                ob_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]))
                fo1 = fo_piece[ob_index_list]
                for j in range(len(gle) - 1):
                    fo_index_list = np.where((fo1 >= gle[j]) & (fo1 < gle[j + 1]))
                    conf_mx[j, i] = len(fo_index_list[0])
        else:
            ngrade = len(index_list0)
            conf_mx = np.zeros((ngrade, ngrade))
            for i in range(ngrade):
                ob_index_list = np.where(ob == index_list0[i])
                if(len(ob_index_list)==0):continue
                fo1 = fo_piece[ob_index_list]
                for j in range(ngrade):
                    fo_index_list = np.where(fo1 == index_list0[j])
                    conf_mx[j, i] = len(fo_index_list[0])
        row_sums = conf_mx.sum(axis=1, keepdims=True)
        conf_mx = np.hstack((conf_mx, row_sums))
        line_sums = conf_mx.sum(axis=0, keepdims=True)
        conf_mx = np.vstack((conf_mx, line_sums))
        index_list = copy.deepcopy(index_list0)
        index_list.append('sum')
        conf_mx_list.append(conf_mx)

        if save_path is not None:
            table_data = pd.DataFrame(conf_mx,
                                      columns=pd.MultiIndex.from_product([['观测'], index_list]),
                                      index=pd.MultiIndex.from_product([['预报'], index_list])
                                      )
            table_data_list.append(table_data)
    if save_path is not None:
        with pd.ExcelWriter(save_path) as writer:
            for i in range(len(table_data_list)):
                table_data_list[i].to_excel(writer, sheet_name=label[i])
        print("列联表已以excel表格形式保存至" + save_path)
    result = np.array(conf_mx_list)
    result = result.squeeze()
    return result


def frequency_table(ob,fo,grade_list=None, member_list = None,save_path=None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :param save_path:
    :param member_list:
    :return:
    '''

    conf_mx_list = []
    table_data_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    label = ["观测"]
    if member_list is None:
        if new_Fo_shape[0] <= 1:
            label.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)

    if grade_list is None:
        new_fo = copy.deepcopy(fo).flatten()
        new_ob = copy.deepcopy(ob).flatten()
        # index_list = list(set(np.hstack((new_ob, new_fo))))
        fo_set = list(set(new_fo.tolist()))
        ob_set = list(set(new_ob.tolist()))
        fo_set.extend(ob_set)
        index_list = list(set(fo_set))
        if len(index_list) > 30:
            print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
            return
    else:
        index_list = ["<" + str(grade_list[0])]
        for index in range(len(grade_list) - 1):
            index_list.append("[" + str(grade_list[index]) + "," + str(grade_list[index + 1]) + ")")
        index_list.append(">=" + str(grade_list[-1]))
    conf_mx = np.zeros((len(label), len(index_list)))
    for line in range(new_Fo_shape[0]):
        fo_piece = new_Fo[line, :]
        if grade_list is not None:
            ngrade = len(grade_list) + 1
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):
                if line == 0:
                    ob_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]))
                    conf_mx[0, i] = len(ob_index_list[0])
                fo_index_list = np.where((fo_piece >= gle[i]) & (fo_piece < gle[i + 1]))
                conf_mx[line + 1, i] = len(fo_index_list[0])
        else:
            for i in range(len(index_list)):
                if line ==0:
                    ob_index_list = np.where(ob == index_list[i])
                    conf_mx[0, i] = len(ob_index_list[0])
                fo_index_list = np.where(fo_piece == index_list[i])
                conf_mx[1+line, i] = len(fo_index_list[0])
    if save_path is not None:
        table_data = pd.DataFrame(conf_mx,
                                  columns=pd.MultiIndex.from_product([['类别'], index_list]),
                                  index=pd.MultiIndex.from_product([['ob-fo'], label])
                                  )
        table_data.to_excel(save_path, sheet_name='sheet1')
        print("频率表已以excel表格形式保存至" + save_path)
    return conf_mx


# -*-coding:utf-8-*-

from .feature_props import feature_props
from . import bearing as bear
from .locperf import *
from .deltametric import *
from .intersect import *
from .utils import get_attributes_for_feat,remove_key_from_list



def feature_comps(look,label_ob,label_fo, which_comps=None,  alpha=0.1, k=1,
                  p=2, c=float('inf'), distfun='distmapfun',
                  deg=True, aty='compass'):

    XtmpAttributes = get_attributes_for_feat(look['grd_ob_features'])
    remove_list = ['Type', 'xrange', 'yrange', 'dim', 'xstep', 'ystep', 'warnings', 'xcol', 'yrow']  # 需要移除的属性列表
    xkeys = remove_key_from_list(list(look['grd_ob_features'].keys()), remove_list)
    X = {"m": look['grd_ob_features'][label_ob]}
    X.update(XtmpAttributes)

    YtmpAttributes = get_attributes_for_feat(look['grd_fo_features'])
    ykeys = remove_key_from_list(list(look['grd_fo_features'].keys()), remove_list)
    Y = {"m": look['grd_fo_features'][label_fo]}
    Y.update(YtmpAttributes)
    grid0 = look["grid"]

    out = {}
    if which_comps is None:
        which_comps = ["cent_dist", "angle_diff", "area_ratio", "int_area", "bdelta", "haus", "ph", "med", "msd", "fom",
                       "minsep", "bearing"]
    id1 = []
    for i, val in enumerate(["cent_dist", "angle_diff", "area_ratio", "int_area", "bearing"]):
        id1.append(val in which_comps)
    if any(id1):
        list1 = []
        if "cent_dist" in which_comps or "bearing" in which_comps:
            list1.append("centroid")
        if "area_ratio" in which_comps or "int_area" in which_comps:
            list1.append("area")
        if "angle_diff" in which_comps:
            list1.append("axis")
    id2 = []
    for i, val in enumerate(["ph", "med", "msd", "fom", "minsep"]):
        id2.append(val in which_comps)
    list2 = []
    # 需要考虑id2为矩阵的情况
    if any(id2):
        list2 = np.array(["ph", "med", "msd", "fom", "minsep"])[id2]
    if any(id1):
        x_single_props = feature_props(look,label_ob,"ob", list1)
        y_single_props = feature_props(look,label_fo,"fo", list1)
        out = {}
        if "cent_dist" in which_comps:
            x_cent_x = x_single_props["centroid"]["x"]
            x_cent_y = x_single_props["centroid"]["y"]
            y_cent_x = y_single_props["centroid"]["x"]
            y_cent_y = y_single_props["centroid"]["y"]
            out['cent_dist'] = math.sqrt((y_cent_x - x_cent_x)**2 + (y_cent_y - x_cent_y)**2)
        if "angle_diff" in which_comps:
            phiX = x_single_props['axis']['OrientationAngle']['MajorAxis'] * math.pi/180
            phiY = y_single_props['axis']['OrientationAngle']['MajorAxis'] * math.pi/180
            out['angle_diff'] = abs(math.atan2(math.sin(phiX - phiY), math.cos(phiX - phiY)) * 180/math.pi)
        if "area_ratio"in which_comps or "int_area" in which_comps:
            Xa = x_single_props['area']
            Ya = y_single_props['area']
        if "area_ratio" in which_comps:
            out['area_ratio'] = min(Xa, Ya)/max(Xa, Ya)
        if "int_area" in which_comps:
            denom = (Xa + Ya)/2
            XY = intersect(X, Y)
            XYa = np.sum(XY["m"]) * (grid0.dlon*grid0.dlat)
            #XYa = (fp.feature_props(XY, None, "area", sizefac**2, None, loc))["area"]
            out['int_area'] = XYa/denom

        if "bearing" in which_comps:
            out['bearing'] = bear.bearing(np.vstack((np.array(y_single_props['centroid']['x']), np.array(y_single_props['centroid']['y']))).transpose(),
                                          np.vstack((np.array(x_single_props['centroid']['x']), np.array(x_single_props['centroid']['y']))).transpose(), deg, aty)
        if "bdelta" in which_comps:
            out['bdelta'] = deltametric(X, Y, p, c)
        if "haus" in which_comps:
            out['haus'] = deltametric(X, Y, float('inf'), float('inf'))
        if any(id2):
            out.update(locperf(X, Y, list2, alpha, k))
        return out
'''
if __name__ == '__main__':
    grd_ob = grd_ob_feature
    grd_fo = grd_fo_feature
    
'''



import os
import meteva
import time
from meteva.base.io.GDS_data_service import GDSDataService
from meteva.base.io import DataBlock_pb2




def copy_data_one_cup(input_path,output_path,read_method,write_method,read_args,write_args):

    dat = read_method(filename = input_path,**read_args)
    meteva.base.creat_path(output_path)
    write_method(dat,save_path = output_path,**write_args)



def copy_data(input_root_dir,output_root_dir,read_method,write_method,read_args ={},write_args ={}, recover = False,tran_extension = None,process_count = 1):
    #获取所以的输入文件
    input_root_dir = input_root_dir.replace("\\", "/")
    output_root_dir = output_root_dir.replace("\\", "/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(input_root_dir)
    len_input = len(input_root_dir)
    input_path_list = []
    output_path_list = []
    for file in file_list:
        filename_part = file[len_input:]
        output_path = output_root_dir + filename_part
        if tran_extension is not None:
            if tran_extension[0] == "+":
                output_path += tran_extension[1:]
            elif tran_extension[0] == "-":
                str_len = len(tran_extension) - 1
                if output_path[-str_len:] == tran_extension[1:]:
                    output_path = output_path[:-str_len]
            else:
                "print the chang_file_type args must start with + or -"
        if recover or not os.path.exists(output_path):
            input_path_list.append(file)
            output_path_list.append(output_path)
    meteva.base.tool.multi_run(process_count = process_count,method=copy_data_one_cup,
                               Input_path = input_path_list,Output_path = output_path_list,
                               read_method = read_method,write_method = write_method,
                               read_args  = read_args,write_args = write_args)



def copy_m4_to_nc(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0

    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file+".nc"
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_from_micaps4(path_input)
            else:
                grd = meteva.base.io.read_griddata_from_micaps4(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_nc(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                #print(path_output)



def copy_gds_to_nc(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file+".nc"
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_from_gds_file(path_input)
            else:
                grd = meteva.base.io.read_griddata_from_gds_file(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_nc(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)


def copy_gds_to_m4(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_from_gds_file(path_input)
            else:
                grd = meteva.base.io.read_griddata_from_gds_file(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_micaps4(grd,path_output,effectiveNum = effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)



def copy_wind_gds_to_nc(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file+".nc"
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_gridwind_from_gds_file(path_input)
            else:
                grd = meteva.base.io.read_gridwind_from_gds_file(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_nc(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)



def copy_wind_gds_to_m11(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_wind_from_gds_file(path_input)
            else:
                grd = meteva.base.io.read_gridwind_from_gds_file(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_micaps11(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)



def copy_wind_m2_to_m11(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_wind_from_micaps2(path_input)
            else:
                grd = meteva.base.io.read_griddata_wind_from_micaps2(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_micaps11(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1

                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60)) + 1
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)



def copy_wind_m11_to_nc(input_root_dir,output_root_dir,effectiveNum = 3,recover= False,grid = None):
    input_root_dir = input_root_dir.replace("\\","/")
    output_root_dir = output_root_dir.replace("\\","/")
    if input_root_dir[-1] != "/":
        input_root_dir += "/"
    if output_root_dir[-1] != '/':
        output_root_dir += "/"
    (gds_dir,file_model) = os.path.split(input_root_dir)
    save_dir = output_root_dir
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(gds_dir)
    len_input = len(input_root_dir)
    file_num = len(file_list)
    start = time.time()
    copyed_num = 0
    for i in range(file_num):
        file = file_list[i]
        path_input = file.replace("\\","/")
        path_file = path_input[len_input:]
        path_output = output_root_dir + path_file+".nc"
        if not os.path.exists(path_output) or recover:
            meteva.base.tool.path_tools.creat_path(path_output)
            if grid is None:
                grd = meteva.base.io.read_griddata_wind_from_micap11(path_input)
            else:
                grd = meteva.base.io.read_gridwind_from_micap11(path_input,grid=grid)
            if grd is not None:
                meteva.base.io.write_griddata_to_nc(grd,path_output,effectiveNum)
                end = time.time()
                copyed_num +=1
                left_minutes = int((end - start) * ( file_num - i -1) / (copyed_num * 60))
                print("剩余" + str(left_minutes) + "分钟")
                print(path_output)


def download_gds_files_to_local(ip, port, gds_dir, local_dir,recover= False):
    print(gds_dir)
    filelist = meteva.base.tool.path_tools.get_gds_file_list_in_one_dir(ip, port, gds_dir)
    for file in filelist:
        save_path = local_dir +  "/" + file
        if not os.path.exists(save_path) or recover:
            service = GDSDataService(ip, port)
            if service is not None:
                status, response = service.getData(gds_dir, file)
                if status == 200:
                    ByteArrayResult = DataBlock_pb2.ByteArrayResult()
                    ByteArrayResult.ParseFromString(response)
                    meteva.base.tool.path_tools.creat_path(save_path)
                    br = open(save_path, 'wb')
                    br.write(ByteArrayResult.byteArray)
                    br.close()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 10 10:54:21 2021

"""
import numpy as np
import pandas as pd
#import pyreadr
import sys

#sys.path.append(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/lib')
from meteva.method.space.field_significance import LocSig, is_sig

# import matplotlib.pyplot as plt

def spatbiasFS(X, Y, loc='NULL', block_length='NULL', alpha_boot=0.05,
               field_sig=0.05, bootR=1000, ntrials=1000, show=False):
    out = {}
    '''
    if (!is.null(loc)) {
        data.name <- c(as.character(substitute(X)), as.character(substitute(Y)), 
                   as.character(substitute(loc)))
        names(data.name) <- c("verification", "forecast", "locations")
        }
    else {
        data.name <- c(as.character(substitute(X)), as.character(substitute(Y)))
        names(data.name) <- c("verification", "forecast")
        }
    out$data.name <- data.name
    '''
    errfield = Y - X
    hold = LocSig.LocSig(Z=errfield, numrep=bootR, block_length=block_length,
                         alpha=alpha_boot)
    res = is_sig.is_sig(errfield, hold, n=ntrials, fld_sig=field_sig, show=show)
    out['block_boot_results'] = hold
    out['sig_results'] = res
    out['field_significance'] = field_sig
    out['alpha_boot'] = alpha_boot
    out['bootR'] = bootR
    out['ntrials'] = ntrials
    out['class'] = "spatbiasFS"

    return out


if __name__ == '__main__':
    # 导入数据，并裁切一部分
    # obsData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMobsEx.rda')[
    #     'GFSNAMobsEx']
    # fcstData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMfcstEx.rda')[
    #     'GFSNAMfcstEx']
    # locData = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/fieldSignficance/data/GFSNAMlocEx.rda')[
    #     'GFSNAMlocEx']
    # locId = (locData["Lon"] >= -90) & (locData["Lon"] <= -75) & (locData["Lat"] <= 40)
    # locSub = locData[locId]
    # GFSobsSub = obsData.T[locId].T
    # GFSfcstSub = fcstData.T[locId].T

    ob = np.random.randn(100,400)
    fo = np.random.randn(100,400) + 0.1
    # 调用示例
    lookFS = spatbiasFS(X=ob, Y=fo)

    '''
    #参数赋值
    X = GFSobsSub
    Y = GFSfcstSub
    loc=locSub
    block_length = 'NULL'
    alpha_boot = 0.05
    field_sig = 0.05
    bootR = 500
    ntrials=500
    show = True
    '''






level ="level"
time = "time"
time_range = "time_range"
year = "year"
month = "month"
day = "day"
dayofyear = "dayofyear"
hour = "hour"
ob_time = "ob_time"
ob_time_range = "ob_time_range"
ob_year = "ob_year"
ob_month = "ob_month"
ob_day = "ob_day"
ob_dayofyear = "ob_dayofyear"
ob_hour = "ob_hour"
dtime = "dtime"
dtime_range = "dtime_range"
dday = "dday"
dhour = "dhour"
id = "id"
lon = "lon"
lon_range = "lon_range"
lon_step = "lon_step"
lat = "lat"
lat_range = "lat_range"
lat_step = "lat_step"
last = "last"
last_range = "last_range"
last_step = "last_step"
grid = "grid"
value = "value"
drop_IV = "drop_IV"
gxy = "gxy"
gxyz = "gxyz"


# -*-coding:utf-8-*-

from .distmap import *
from .sma import sma
from . import angles_psp
from . import midpoints_psp
from . import as_psp


def convexHull(rx0):
    max_v = np.max(rx0)
    min_v = np.min(rx0)
    d = rx0[:,0] + (rx0[:,1] - min_v)/max_v
    rx = rx0[d.argsort()]
    #先求下包络线
    # 首先用一根皮筋从头连到尾
    line_low = rx.copy().tolist()
    len0 = -1
    while len(line_low) != len0:
        len0 = len(line_low)
        line_list = [line_low[0]]
        for i in range(1,len0-1):
            dx1 = line_low[i][0] - line_low[i-1][0]
            dy1 = line_low[i][1] - line_low[i-1][1]
            dx2 = line_low[i+1][0] - line_low[i - 1][0]
            dy2 = line_low[i+1][1] - line_low[i - 1][1]
            cross = dx1 * dy2 - dy1 * dx2
            if cross>0:
                line_list.append(line_low[i])  # 相邻两个点是逆时针选择则保留，否则松开
        line_list.append(line_low[-1])
        line_low = line_list

    #再求上包络线
    # 首先用一根皮筋从头连到尾
    line_up = rx.tolist()
    len0 = -1
    while len(line_up) != len0:
        len0 = len(line_up)
        line_list = [line_up[0]]
        for i in range(1,len0-1):
            dx1 = line_up[i][0] - line_up[i-1][0]
            dy1 = line_up[i][1] - line_up[i-1][1]
            dx2 = line_up[i+1][0] - line_up[i - 1][0]
            dy2 = line_up[i+1][1] - line_up[i - 1][1]
            cross = dx1 * dy2 - dy1 * dx2
            if cross<0:
                line_list.append(line_up[i])  # 相邻两个点是顺时针选择则保留，否则松开
        line_list.append(line_up[-1])
        line_up = line_list

    #将上下两根包络线想连

    line_all = []
    for i in range(len0-1,0,-1):
        line_all.append(line_up[i])
    line_all.extend(line_low[0:-1])

    result = np.array(line_all).astype(np.float32)
    return result

def caculate_feature_axis(pts,fac = 1, twixt = False):
    axfit_frame = {'x': pts[:, 0], 'y': pts[:, 1]}
    axfit = sma(axfit_frame)
    axis_x = np.array([axfit['from'], axfit['to']])
    # a = axfit['coef']['slope']
    # b = axfit['coef']['intercept']
    a = axfit["slope"]
    b = axfit["intercept"]
    axis_y = b + a * axis_x

    if axis_x[0] is None or axis_x[1] is None or axis_y[0] is None or axis_y[1] is None:
        return None
    # axwin = owin(xrange = range(axis_x), yrange = range(axis_y))
    axis_frame = {'x0': axis_x[0], 'y0': axis_y[0], 'x1': axis_x[1], 'y1': axis_y[1]}
    # MajorAxis = as.psp(pd.DataFrame(axis_frame), window=axwin)
    theta = angles_psp.angles_psp(axis_frame)
    if 0 <= theta <= math.pi / 2:
        theta2 = math.pi / 2 - theta
    else:
        theta2 = 3 * math.pi / 2 - theta
    tmp = rotate(pts, theta2)
    tmp = boundingbox(tmp, tmp)
    l = float(tmp['x_range'][1] - tmp['x_range'][0])
    theta = theta * 180 / math.pi
    if twixt:
        if 90 < theta <= 270:
            theta = theta - 180
        elif 270 < theta <= 360:
            theta = theta - 360
    MidPoint = midpoints_psp.midpoints_psp(axis_frame)
    #r = lengths_psp.lengths_psp(axis_frame) * fac
    r = float(tmp["y_range"][1] - tmp["y_range"][0])
    phi = angles_psp.angles_psp(rotate(np.array([[axis_x[0], axis_y[0]], [axis_x[1], axis_y[1]]]), math.pi / 2))
    minor_frame = {'xmid': MidPoint['x'], 'ymid': MidPoint['y'], 'length': l/fac, 'angle': phi}
    MinorAxis = as_psp.as_psp(minor_frame)
    phi = phi * 180 / math.pi
    out = {}
    out['phi'] = phi
    out['MajorAxis'] = {'ends': axis_frame}
    out['MinorAxis'] = MinorAxis
    out['OrientationAngle'] = {'MajorAxis': theta, 'MinorAxis': phi}
    out['aspect_ratio'] = l / r
    out["MidPoint"] = {}
    out['MidPoint']["x"] = MidPoint["x"]
    out["MidPoint"]["y"] = MidPoint["y"]
    out["window"] = MidPoint["window"]
    out['lengths'] = {'MajorAxis': r, 'MinorAxis': l}
    out['sma_fit'] = axfit
    return out

def feature_axis(look,label,ob_or_fo = None,fac = 1, flipit=False, twixt=False):

    grid0 = look["grid"]
    if ob_or_fo =="ob":
        tmp = look['grd_ob_features'][label]
    elif ob_or_fo == "fo":
        tmp = look['grd_fo_features'][label]
    else:
        if label in look["grd_features"].keys():
            tmp = look['grd_features'][label]
        else:
            return None
    x = np.zeros((grid0.nlat,grid0.nlon))
    x[tmp] = 1
    out = {}
    if flipit:
        x = np.transpose(x)

    rx = getRedDotsCoordinatesFromLeftToRight(x)
    pts = convexHull(rx)
    out['point'] = rx
    out["point"][:,0] = grid0.slon +  out["point"][:,0] * grid0.dlon
    out["point"][:,1] = grid0.slon + out["point"][:,1] * grid0.dlon

    out['pts'] = pts
    out["pts"][:,0] = grid0.slon + out["pts"][:,0] * grid0.dlon
    out["pts"][:,1] = grid0.slat + out["pts"][:,1] * grid0.dlat

    out1 = caculate_feature_axis(pts,fac=fac,twixt=twixt)
    out.update(out1)

    return out

'''
if __name__ == '__main__':
    data = np.load("../../centmatchResult_PA3.npy", allow_pickle=True).tolist()
    XtmpAttributes = utils.get_attributes_for_feat(data['Xlabelsfeature'])
    remove_list = ['Type', 'xrange', 'yrange', 'dim', 'xstep', 'ystep', 'warnings', 'xcol', 'ycol']
    xkeys = utils.remove_key_from_list(list(data['Xlabelsfeature'].keys()), remove_list)
    Xtmp = {"m": data['Xlabelsfeature']['labels_5']}
    Xtmp.update(XtmpAttributes)
    look_feature_axis = feature_axis(Xtmp)

#data = np.load(r'F:\Work\MODEtra_test\FeatureFinder\deltammResult_PA3.npy', allow_pickle = True).tolist()
data = look_deltamm.copy()
XtmpAttributes = utils.get_attributes_for_feat(data['Xlabelsfeature'])
remove_list = ['Type', 'xrange', 'yrange', 'dim', 'xstep', 'ystep', 'warnings', 'xcol', 'ycol']
xkeys = utils.remove_key_from_list(list(data['Xlabelsfeature'].keys(    )), remove_list)
Xtmp = {"m": data['Xlabelsfeature']['labels_5']}
Xtmp.update(XtmpAttributes)
look_feature_axis = feature_axis(Xtmp)
'''


# -*- coding: utf-8 -*-
"""
Created on Tue Jun  9 11:40:04 2020

@author: 1
"""

#======================  make.SpatialVx()  =======================

import numpy as np
import sys


def isAllDigital(array):      #这个函数是用来判断list是否为一维纯数值，如果是二维，返回false
    ret = True
    for v in array:
        ret = ret and isinstance(v,(int,float,np.int,np.int32))
        if not ret:
            return False
    return ret


def makeSpatialVx(X, Xhat, loc, thresholds = [None, None], projection = False, 
                  subset = None, timevals = None,reggrid = True, Map = False, 
                  locbyrow = False, fieldtype = "", units = "", dataname = "", 
                  obsname = "X", modelname = "Xhat", q = (0, 0.1, 0.25, 0.33, 0.5, 0.66, 0.75, 0.9, 0.95), 
                  qs = None ):
    #参数：X, Xhat, threshold, loc, projection, subset, timevals,reggrid, map, locbyrow, fieldtype, units, dataname, obsname, modelname, q, qs
        
    #if (X.shape == None):
    if (type(X) == list):    #判断X是否为列表，X实际为二维矩阵
        nobs = X.shape[0] * X.shape[1]
        xdim = None
    else:
        nobs = 1
        xdim = X.shape
        
    #if (Xhat.shape == None):
    if (type(Xhat) == list):    #判断X是否为列表
        nforecast = Xhat.shape[0] * Xhat.shape[1]
        ydim = None
    else:
        nforecast = 1
        ydim = Xhat.shape
        
    
    if (xdim != ydim):
        try:
            sys.exit(0)
        except:
            print("make.SpatialVx: dim of X{0}  must be the same as dim of Xhat{1} ".format(X.shape,Xhat.shape))
    
    out = {"X": X, "Xhat": Xhat}    #将矩阵存为列表
    
   
    if (all(thresholds) == None):
        if (nobs > 1):    #猜测是输入的观测数据大于一个时次，对每个时次的数据分别求分位数
            othresh = np.zeros((9,1))    #定义othresh为空数组，长度与q一致，9
            for i in range(1, nobs):
                #X_1 = np.array(X)
                #X_flat = X_1.flatten()    #dataframe没有flatten属性，先转array
                quantile_X = np.quantile(X, q)  #计算X的分位数
                othresh = np.hstack([quantile_X, othresh])
        else:
            othresh = np.quantile(X, q)    #数据大小为（9，1）是写死的，因为所要求的的分位数的个数是确定的
        
        if (nforecast > 1):    #猜测是输入的观测数据大于一个时次，对每个时次的数据分别求分位数
            fthresh = np.zeros()    #定义fthresh为空数组，长度未知
            for i in range(1, nforecast):
                quantile_Xhat = np.quantile(Xhat, q)  #计算Xhat的q分位数
                fthresh = np.hstack([fthresh, quantile_Xhat])  #原函数里面直接生成的矩阵，没有赋值到任何变量
        else:
            fthresh = np.quantile(Xhat, q)    #数据大小为（9，1）是写死的，因为所要求的的分位数的个数是确定的
        qs = str(q)
        #X = othresh
        #Xhat = fthresh
        thresholds = [othresh, fthresh]
        
    elif (type(thresholds) == list):    #判断thresholds是否为一个list
        threshnames = ["X","Xhat"]    #获取列表的名称
        if (("X"not in threshnames) or ("Xhat" not in threshnames)):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.  List must have components named X and Xhat.")
       
        odim = np.array([len(thresholds), np.ndim(thresholds)])    #thresholds 第一项(X)的数组长度赋值给odim
        fdim = np.array([len(thresholds), np.ndim(thresholds)])    #thresholds 第二项(Xhat)的数组长度赋值给fdim
        
    
        if(odim[0] != fdim[0]):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.  X must have same number of thresholds (rows) as Xhat.")
        nth = odim[0]
        
        if(any(odim) == None):
            thresholds[0] = np.array(thresholds[0]).reshape(len(thresholds[0]),nobs)
        elif(odim[1] == 1 and nobs > 1):
            thresholds[0] = np.array(thresholds[0]).reshape(len(thresholds[0]),nobs)
        elif (odim[1] > 1 and odim[1] != nobs):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.")
        threshold = 'threshold'
        qs = []
        for i in range(nth):
            qs.append(threshold +' '+ str(i))
       
        
    elif(type(thresholds) == list):    #判断thresholds是否为vector,R里面检查是否最多只有一个属性：name。即查看其属性，如果没有属性或者只有一个name属性，才返回TRUE

        qs = str(thresholds)
        nth = len(thresholds)
        thresholds = [np.array(thresholds).reshape(nth,nobs),np.array(thresholds).reshape(nth,nforecast)]
        #else stop("make.SpatialVx: invalid thresholds argument.  Must be a vector or list.")
    else:
        try:
            sys.exit(0)
        except:
            print("make.SpatialVx: invalid thresholds argument.  Must be a vector or list.")
        
    if (loc.any() == None):    #判断loc是否为空,如果为空的话，生成坐标信息
        rep00_1 = np.arange(1, xdim[0], 1)
        rep00 = np.tile(rep00_1, xdim[1])
        rep0_1 = np.arange(1, xdim[1], 1)
        rep0 = rep0_1.repeat(xdim[0], axis = 0)    #按行进行元素重复
        loc = np.vstack((rep00, rep0))
    if (timevals == None):
        if (len(xdim) == 3):
            timevals = np.arnge(1, xdim[2])
        else:
            timevals = 1
    if (dataname != None):
        msg = dataname
    else :
        msg = ""
    if (fieldtype != "" and units != ""):
        msg = msg + '\n' + fieldtype +' ' + '(' + units + ')'
    elif (fieldtype != ""):
        msg = msg + '\n' + fieldtype
    elif (units != ""):
        msg = msg + ' ' + '(' + units + ')'
        
       
    out = {"X": X, "Xhat": Xhat, "class":"SpatialVx", "xdim":xdim, "time":timevals,\
               "thresholds":thresholds, "loc":loc, "locbyrow":locbyrow, \
               "subset":subset, "dataname":dataname, "obsname":obsname, \
               "modelname":modelname, "nobs":nobs, "nforecast":nforecast, \
               "fieldtype":fieldtype , "units":units , "projection":projection , \
               "reggrid":reggrid , "Map":Map , "qs":qs, "msg":msg}
    return out

'''

hold = makeSpatialVx(X = pd.read_csv(r'F:\Work\MODE\tra_test\QPEF\QPE\0802.csv'),
                     Xhat = pd.read_csv(r'F:\Work\MODE\tra_test\QPEF\QPF\0801.csv'),
                     thresholds = [0.01, 20.01], loc = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\ICPg240Locs.csv"),
                     projection = True, subset = None, timevals = None, reggrid = True, 
                     Map = True, locbyrow = True, fieldtype = "Precipitation", units = ("mm/h"), 
                     dataname = "ICP Perturbed Cases", obsname = "pert000" ,modelname = "pert004",
                     q = (0, 0.1, 0.25, 0.33, 0.5, 0.66, 0.75, 0.9, 0.95) ,qs = None)            
#loc参数最好不要为空,否则需要多一些参数重新生成
hold = makeSpatialVx(X = pd.read_csv(r'F:\Work\MODE\tra_test\QPEF\QPE\0802.csv'),
                     Xhat = pd.read_csv(r'F:\Work\MODE\tra_test\QPEF\QPF\0801.csv'), 
                     loc = pd.read_csv("F:\\Work\\MODE\\tra_test\\FeatureFinder\\ICPg240Locs.csv"),
                     fieldtype = "Precipitation")
'''           





















import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签
plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
import numpy as np
import copy
import meteva


def box_plot_ensemble(ob, fo,member_list = None, vmax = None,vmin = None,save_path=None,show = False,dpi = 300,title ="频率对比箱须图",
                      sup_fontsize = 10,width = None,height = None):
    '''
    box_plot 画一两组数据的箱型图
    ---------------
    :param ob:实况数据 一维的numpy
    :param fo:预测数据 二维的numpy数组
    :param save_path 不为None时输出到图片中
    :return: 无
    '''
    en_num = fo.shape[0]
    if width is None:
        width = en_num * 0.22 + 0.3
        if width < 4:
            width = 4
        if width >8:
            width = 8
    if height is None:
        height = 0.6 * width
    data = np.zeros((len(ob),en_num+1))
    data[:,0] = ob[:]
    data[:,1:] = fo[:,:].T

    fig = plt.figure(figsize=(width,height),dpi = dpi)
    #plt.boxplot((observed, forecast), labels=["观测","预报" ])


    if member_list is None:
        labels = ["ob\n观测"]
        for i in range(en_num):
            if i == int(en_num/2):
                labels.append(str(i+1)+"\n预报")
            else:
                labels.append(str(i+1))
    else:
        labels = ["观测"]
        labels.extend(member_list)

    bplot = plt.boxplot((data),showfliers =True,patch_artist=True,labels=labels,sym = '.')
    for i, item in enumerate(bplot["boxes"]):
        if i == 0:
            item.set_facecolor("pink")
        else:
            item.set_facecolor("lightblue")
    plt.axvline(0.5,color = "b")
    plt.subplots_adjust(left=0.5/width,right=1-0.1/width)
    plt.xticks(fontsize = 0.9 * sup_fontsize)
    plt.yticks(fontsize = 0.8 * sup_fontsize)
    plt.title(title,fontsize = sup_fontsize)
    plt.ylabel("value")
    if vmin is not None or vmax is not None:
        if vmin is not None:
            if vmax is None:
                vmax = np.max(data)
                dmax = vmax - vmin
                plt.ylim(vmin,vmax+ dmax * 0.05)
            else:
                plt.ylim(vmin, vmax)
        else:
            vmin = np.min(data)
            dmax = vmax - vmin
            plt.ylim(vmin- dmax * 0.05,vmax)

    if save_path is None:
        show = True
    else:
        meteva.base.creat_path(save_path)
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()


def rank_histogram(ob,fo,vmax = None,save_path= None,show = False,dpi = 300,title = "等级柱状图",
                   sup_fontsize=10, width=None, height=None):
    '''
    :param ob:一维numpy数组
    :param fo: 二维numpy数组
    :param save_path:
    :return:
    '''
    en_num = fo.shape[0]
    sample_num = ob.size
    fo1 = copy.deepcopy(fo)
    fo1.sort(axis = 0)
    index = np.where(ob<fo1[0,:])

    en_num = fo.shape[0]
    if width is None:
        width = en_num * 0.22 + 0.3
        if width < 6:
            width = 6
        if width >10:
            width = 10
    if height is None:
        height = width * 0.4

    fig = plt.figure(figsize=(width,height),dpi = dpi)

    rank_num = [len(index[0])]
    for i in range(en_num-1):
        index = np.where((ob>=fo1[i,:]) & (ob < fo1[i+1,:]))
        rank_num.append(len(index[0]))
    index = np.where(ob>=fo1[-1,:])

    rank_num.append(len(index[0]))
    rank_rate = np.array(rank_num)/sample_num
    x = np.arange(0,en_num+1)

    plt.xticks(x,fontsize = 0.8 * sup_fontsize)
    plt.bar(x,rank_rate)
    plt.ylabel("比例",fontsize = 0.9 * sup_fontsize)
    plt.xlabel("观测值在集合序列中的排序号",fontsize = 0.9 * sup_fontsize)
    plt.title(title,fontsize = sup_fontsize)
    if vmax is None:
        ymax = np.max(rank_rate) * 1.5
    else:
        ymax = vmax
    plt.ylim(0,ymax)
    plt.xlim(-0.5,en_num+0.5)
    if save_path is None:
        show = True
    else:
        meteva.base.creat_path(save_path)
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()





import meteva
import numpy as np
from meteva.base import IV
from meteva.method.yes_or_no.score import ts_hfmc, ets_hfmc, bias_hfmc, far_hfmc, mr_hfmc,pod_hfmc,sr_hfmc,pofd_hfmc
import math


def tc(ob, fo, grade_list=None):
    '''
    accuracy 求多分类预报准确率
    :param ob: 实况数据 不确定维numpy
    :param fo:  预测数据 不确定维numpy
    :param grade_list:等级，如果grade_list= None则ob和fo里的值代表类别，否则，根据grade_list来进行分类
    :return: 返回一维数组，包括（总样本数，正确数）
    '''
    tc_array_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):
        # 多分类预报准确率
        if grade_list is None:
            # ob 和fo是不确定维的numpy数组，其中每个值是代表分类的类别
            ob1 = ob.reshape((-1))
            fo1 = new_Fo[line, :].reshape((-1))
        else:
            ob1 = np.zeros_like(ob)
            fo1 = np.zeros_like(new_Fo[line, :])
            # ob 和fo 是连续的变量，通过 threshold_list 将ob 和fo划分成连续的等级之后再计算等级准确性
            for index in range(len(grade_list) - 1):
                ob_index_list = np.where((grade_list[index] <= ob) & (ob < grade_list[index + 1]))
                ob1[ob_index_list] = index + 1
                fo_index_list = np.where(
                    (grade_list[index] <= new_Fo[line, :]) & (new_Fo[line, :] < grade_list[index + 1]))
                fo1[fo_index_list] = index + 1
            ob_index_list = np.where(grade_list[-1] <= ob)
            ob1[ob_index_list] = len(grade_list)
            fo_index_list = np.where(grade_list[-1] <= new_Fo[line, :])
            fo1[fo_index_list] = len(grade_list)
        correct_num = np.sum(fo1 == ob1)

        tc_array_list.append(np.array([ob.size, correct_num]))
    tc_array = np.array(tc_array_list)
    shape = list(Fo_shape[:ind])
    shape.append(2)
    tc_array = tc_array.reshape(shape)
    return tc_array


def tcof(ob, fo, grade_list=None):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    tcof_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    if (grade_list is None):
        fo_list = list(set(fo.reshape((-1)).tolist()))
        ob_list = list(set(fo.reshape((-1)).tolist()))
        fo_list.extend(ob_list)
        grade_list = list(set(fo_list))
        grade_list.sort()
        if len(grade_list) > 30:
            print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
            return
    for line in range(new_Fo_shape[0]):
        tc1 = tc(ob, new_Fo[line, :],grade_list=grade_list)
        ft = meteva.method.multi_category.table.frequency_table(ob, new_Fo[line, :], grade_list=grade_list)
        tcof1 = np.zeros((len(grade_list) + 2, 2))
        tcof1[0, :] = tc1
        tcof1[1:, :] = ft.T
        tcof_list.append(tcof1)

    tcof_array = np.array(tcof_list)

    shape = list(Fo_shape[:ind])
    shape_list = [len(grade_list) + 2, 2]
    shape.extend(shape_list)
    tcof_array = tcof_array.reshape(shape)

    return tcof_array


def hfmc_grade(ob, fo, grade_list):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    hfmc_multi_list = []
    Ob_shape = ob.shape
    Fo_shape = fo.shape

    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return

    total_count = ob.size
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    new_grade_list = None
    for line in range(new_Fo_shape[0]):

        new_grade_list = grade_list
        hfmc_m = np.zeros((len(grade_list) + 1, 4))
        gle = [-1e300]
        gle.extend(grade_list)
        gle.append(1e300)
        for i in range(len(gle) - 1):
            #命中样本
            hit_index_list = np.where(
                (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
            hfmc_m[i, 0] = len(hit_index_list[0])

            #空报样本，预报处于某个区间，而观测低于该区间
            fal_index_list = np.where(
                (ob < gle[i]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
            hfmc_m[i, 1] = len(fal_index_list[0])

            # 漏报样本，观测处于某个区间，而预测低于该区间
            mis_index_list = np.where(
                (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] < gle[i]))
            hfmc_m[i, 2] = len(mis_index_list[0])

            #正确无
            nd_list = np.where((ob < gle[i]) & (new_Fo[line, :] < gle[i]))
            hfmc_m[i, 3] = len(nd_list[0])
        hfmc_multi_list.append(hfmc_m)
    hfmc_multi_array = np.array(hfmc_multi_list)
    shape = list(Fo_shape[:ind])
    a = len(new_grade_list)
    if (grade_list is not None):
        a+=1
    shape.append(a)
    shape.append(4)
    hfmc_multi_array = hfmc_multi_array.reshape(shape)
    return hfmc_multi_array


def hfmc_multi(ob, fo, grade_list=None):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    hfmc_multi_list = []
    Ob_shape = ob.shape
    Fo_shape = fo.shape

    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return

    total_count = ob.size
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    new_grade_list = None
    for line in range(new_Fo_shape[0]):

        if (grade_list is None):
            grade_list1 = list(set(ob.flatten()))
            grade_list2 = list(set(fo.flatten()))
            grade_list1.extend(grade_list2)
            new_grade_list = list(set(grade_list1))
            new_grade_list.sort()
            if len(new_grade_list) > 30:
                print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
                return
            hfmc_m = np.zeros((len(new_grade_list), 4))
            for i in range(len(new_grade_list)):
                grade = new_grade_list[i]
                hit_index_list = np.where((ob == grade) & (new_Fo[line, :] == grade))

                hfmc_m[i, 0] = len(hit_index_list[0])
                fal_index_list = np.where((ob != grade) & (new_Fo[line, :] == grade))
                hfmc_m[i, 1] = len(fal_index_list[0])
                mis_index_list = np.where((ob == grade) & (new_Fo[line, :] != grade))
                hfmc_m[i, 2] = len(mis_index_list[0])
                hfmc_m[i, 3] = total_count - hfmc_m[i, 0] - hfmc_m[i, 1] - hfmc_m[i, 2]
        else:
            new_grade_list = grade_list
            hfmc_m = np.zeros((len(grade_list) + 1, 4))
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):

                #命中样本
                hit_index_list = np.where(
                    (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
                hfmc_m[i, 0] = len(hit_index_list[0])

                #空报样本，预报处于某个区间，而观测低于该区间
                fal_index_list = np.where(((ob < gle[i]) | (ob >= gle[i + 1])) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
                hfmc_m[i, 1] = len(fal_index_list[0])
                # 漏报样本，观测处于某个区间，而预测低于该区间
                mis_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]) & ((new_Fo[line, :] < gle[i]) | (new_Fo[line, :] >= gle[i + 1])))
                hfmc_m[i, 2] = len(mis_index_list[0])

                #正确无
                hfmc_m[i, 3] = total_count - hfmc_m[i, 0] - hfmc_m[i, 1] - hfmc_m[i, 2]
        hfmc_multi_list.append(hfmc_m)
    hfmc_multi_array = np.array(hfmc_multi_list)
    shape = list(Fo_shape[:ind])
    a = len(new_grade_list)
    if (grade_list is not None):
        a+=1
    shape.append(a)
    shape.append(4)
    hfmc_multi_array = hfmc_multi_array.reshape(shape)

    return hfmc_multi_array


def accuracy(ob, fo, grade_list=None):
    '''
    accuracy 求多分类预报准确率
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return: 0-1的实数，0代表无技巧，最优预报为1
    '''
    tc1 = tc(ob,fo,grade_list)
    total_count = tc1[...,0]
    correct_count = tc1[...,1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def accuracy_tc(tc_array):
    '''
    :param tc_array:
    :return:
    '''
    total_count = tc_array[..., 0]
    correct_count = tc_array[..., 1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def accuracy_tcof(tcof_array):
    '''

    :param tcof_array:
    :return:
    '''
    total_count = tcof_array[..., 0, 0]
    correct_count = tcof_array[..., 0, 1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def ts_grade(ob,fo,grade_list):
    '''
    分级ts评分
    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    ts_array = ts_hfmc(hfmc_array)
    return ts_array


def ets_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    ets_array = ets_hfmc(hfmc_array)
    return ets_array


def bias_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    bias_array = bias_hfmc(hfmc_array)
    return bias_array


def mr_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    mr_array = mr_hfmc(hfmc_array)
    return mr_array


def far_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    far_array = far_hfmc(hfmc_array)
    return far_array

def pod_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    pod_array = pod_hfmc(hfmc_array)
    return pod_array

def pofd_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    pofd_array = pofd_hfmc(hfmc_array)
    return pofd_array

def sr_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    sr_array = sr_hfmc(hfmc_array)
    return sr_array


def ts_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    ts_array = ts_hfmc(hfmc_array)
    return ts_array


def ets_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    ets_array = ets_hfmc(hfmc_array)
    return ets_array


def bias_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    bias_array = bias_hfmc(hfmc_array)
    return bias_array


def mr_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    mr_array = mr_hfmc(hfmc_array)
    return mr_array


def far_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    far_array = far_hfmc(hfmc_array)
    return far_array


def pod_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    pod_array = pod_hfmc(hfmc_array)
    return pod_array

def pofd_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    pofd_array = pofd_hfmc(hfmc_array)
    return pofd_array

def sr_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    sr_array = sr_hfmc(hfmc_array)
    return sr_array


def hss(ob, fo, grade_list=None):
    '''
    hss heidke技能得分,它表现实际的预报的分类准确性相对于随机分类达到的准确性的技巧
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return:
    '''
    Ob_shape = ob.shape
    Fo_shape = fo.shape
    hss_list = []
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')

        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):

        conf_mx = meteva.method.multi_category.table.contingency_table_multicategory(ob, new_Fo[line, :], grade_list)

        accuracy_score = accuracy(ob, new_Fo[line, :], grade_list)
        total_num = ob.size
        NF_array = conf_mx[0:-1, -1]
        NO_array = conf_mx[-1, 0:-1]
        random_score = np.dot(NF_array, NO_array) / (total_num * total_num)
        if (random_score == 1):
            HSS = IV
        else:
            HSS = (accuracy_score - random_score) / (1 - random_score)
        hss_list.append(HSS)
    if len(hss_list)==1:
        hss_array = hss_list[0]
    else:
        hss_array = np.array(hss_list)
        shape = list(Fo_shape[:ind])
        hss_array = hss_array.reshape(shape)

    return hss_array


def hk(ob, fo, grade_list=None):
    '''
    hk Hanssen和Kuipers判别
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return:
    '''
    # 多分类预报hk技巧评分
    Ob_shape = ob.shape
    Fo_shape = fo.shape
    hk_list = []
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')

        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):
        conf_mx = meteva.method.multi_category.table.contingency_table_multicategory(ob, new_Fo[line, :], grade_list)
        accuracy_score = accuracy(ob, new_Fo[line, :], grade_list)
        total_num = ob.size
        NF_array = conf_mx[0:-1, -1]
        NO_array = conf_mx[-1, 0:-1]
        random_score = np.dot(NF_array, NO_array) / (total_num * total_num)
        ob_rate = np.dot(NO_array, NO_array) / (total_num * total_num)
        HK = (accuracy_score - random_score) / (1 - ob_rate)
        hk_list.append(HK)
    if len(hk_list)==1:
        hk_array = hk_list[0]
    else:
        hk_array = np.array(hk_list)
        shape = list(Fo_shape[:ind])
        hk_array = hk_array.reshape(shape)

    return hk_array


def hk_tcof(tcof_array):
    '''
    :param tcof_array:
    :return:
    '''

    tc_sum = tcof_array[..., 0, 0]
    cc_sum = tcof_array[..., 0, 1]
    accuracy_score = cc_sum / tc_sum
    obn_sum = tcof_array[..., 1:, 0]
    fon_sum = tcof_array[..., 1:, 1]
    dotobfo = np.zeros_like(tc_sum)
    dotobob = np.zeros_like(tc_sum)
    for g in range(obn_sum.shape[-1]):
        dotobfo += obn_sum[..., g] * fon_sum[..., g]
        dotobob += obn_sum[..., g] * obn_sum[..., g]
    random_score = dotobfo / (tc_sum * tc_sum)
    ob_rate = dotobob / (tc_sum * tc_sum)
    if ob_rate.size == 1:
        if ob_rate == 1:
            HK = IV
        else:
            HK = (accuracy_score - random_score) / (1 - ob_rate)
    else:
        ob_rate[ob_rate == 1] = -1
        HK = (accuracy_score - random_score) / (1 - ob_rate)
        HK[ob_rate == -1] = IV
    return HK


def hss_tcof(tcof_array):
    '''
    :param tcof_list:
    :return:
    '''
    tc_sum = tcof_array[..., 0, 0]
    cc_sum = tcof_array[..., 0, 1]
    accuracy_score = cc_sum / tc_sum
    obn_sum = tcof_array[..., 1:, 0]
    fon_sum = tcof_array[..., 1:, 1]
    dotobfo = np.zeros_like(tc_sum)
    dotobob = np.zeros_like(tc_sum)
    for g in range(obn_sum.shape[-1]):
        dotobfo += obn_sum[..., g] * fon_sum[..., g]
        dotobob += obn_sum[..., g] * obn_sum[..., g]
    random_score = dotobfo / (tc_sum * tc_sum)

    if random_score.size == 1:
        if random_score == 1:
            HSS = IV
        else:
            HSS = (accuracy_score - random_score) / (1 - random_score)
    else:
        random_score[random_score == 1] = -1
        HSS = (accuracy_score - random_score) / (1 - random_score)
        HSS[random_score == -1] = IV

    return HSS


def seeps_ctable(contingency_table_multicategory,p1):

    s_fv = np.array([[0, 1 / (1 - p1), 4 / (1 - p1)],
                     [1 / p1, 0, 3 / (1 - p1)],
                     [1 / p1 + 3 / (2 + p1), 3 / (2 + p1), 0]]) * 0.5

    if len(contingency_table_multicategory.shape) == 2:
        ctm_fv = contingency_table_multicategory
        p_fv = ctm_fv[0:-1, 0:-1] / ctm_fv[-1, -1]
        #print(p_fv)
        seeps_error = np.sum(s_fv * p_fv)
        return seeps_error
    else:
        seeps_error_list=[]
        for line in range(contingency_table_multicategory[0]):
            ctm_fv = contingency_table_multicategory[line,:,:]
            p_fv = ctm_fv[0:-1, 0:-1] / ctm_fv[-1, -1]
            seeps_error = np.sum(s_fv * p_fv)
            seeps_error_list.append(seeps_error)
        seeps_error_array = np.array(seeps_error_list)
        #shape = list(Fo_shape[:ind])
        #seeps_error_array = seeps_error_array.reshape(shape)
        return seeps_error_array


def seeps(Ob, Fo, p1=None, threshold=None):
    '''

    :param Ob:
    :param Fo:
    :param p1:
    :param significant_rain_threshold:
    :return:
    '''

    if p1 is None:
        ob1 = np.sort(Ob.flatten())
        index_p1 = np.where(ob1 <= 0.2)
        p1 = len(index_p1[0]) / ob1.size
        p3 = (1 - p1) / 3
        index_p3 = int(math.ceil((1 - p3) * ob1.size)) - 1
        threshold = ob1[index_p3]
        if threshold < 0.3:
            threshold = 0.3
    if p1 == 1: p1 -= 1e-6
    if p1 == 0: p1 = 1e-6
    ctable = meteva.method.contingency_table_multicategory(Ob, Fo,
                                                  grade_list=[0.2000001, threshold])
    seeps_error = seeps_ctable(ctable,p1)
    return seeps_error



def seeps_skill(Ob, Fo, p1=None, threshold=None):
    '''

    :param Ob:
    :param Fo:
    :param p1:
    :param significant_rain_threshold:
    :return:
    '''
    seeps_array = seeps(Ob,Fo,p1= p1,threshold=threshold)
    skill = 1- seeps_array
    return skill


import meteva
import numpy as np
import pandas as pd
import copy
import datetime
#格点转换为站点
def trans_grd_to_sta(grd):
    '''
    网格数据转站点数据
    :param grd: 网格预报数据
    :return: 站点数据
    '''
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    x = grd['lon'].values
    y = grd['lat'].values
    grid_x, grid_y = np.meshgrid(x, y)
    grid_num = len(x) * len(y)
    sta_all = None
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)

    for i in range(len(levels)):
        for j in range(len(times)):
            for k in range(len(dtimes)):
                dat = np.empty((grid_num, 2+len(members)))
                dat[:, 0] = grid_x.reshape(-1)
                dat[:, 1] = grid_y.reshape(-1)
                for m in range(len(members)):
                    dat[:,2+m] = grd.values[m,i,j,k,:,:].reshape(-1)
                sta = pd.DataFrame(dat,columns=column_list1)
                sta["id"] = np.arange(grid_num)
                sta['time'] = times[j]
                sta['dtime'] = dtimes[k]
                sta['level'] = levels[i]
                sta = sta.reindex(columns=column_list2)
                if (sta is None):
                    sta_all = sta
                else:
                    sta_all = pd.concat([sta_all, sta])

    return sta_all

def trans_sta_to_grd(sta):
    """
    将站点形式的规则网格的数据转化为格点数据
    :param sta:站点数据
    :return:返回格点网格数据
    """
    lons = sta.loc[:,'lon'].values
    lons = list(set(lons))
    lons.sort()
    if len(lons) == 1:
        glon = lons
    else:
        lons = np.array(lons)
        dlons = lons[1:] - lons[:-1]
        dlon = np.min(dlons)
        glon = [lons[0],lons[-1],dlon]



    lats =sta.loc[:,'lat'].values
    lats = list(set(lats))
    lats.sort()
    #print(lats)
    if len(lats) == 1:
        glat = lats
    else:
        lats = np.array(lats)
        dlats = lats[1:] - lats[:-1]
        dlat = np.min(dlats)
        glat = [lats[0],lats[-1],dlat]

    #print(glat)

    times= sta.loc[:,'time'].values
    times = list(set(times))
    times.sort()
    if len(times) == 1:
        gtime = times
    else:
        gtime = [times[0],times[-1],times[1] - times[0]]

    slon = lons[0]
    slat = lats[0]
    dtime_list = list(set(sta['dtime'].values.tolist()))
    level_list = list(set(sta['level'].values.tolist()))
    member_list = sta.columns[6:].tolist()

    grid0 = meteva.base.basicdata.grid(glon, glat,gtime = gtime,
                                                        level_list=level_list,dtime_list=dtime_list,member_list= member_list)

    grd = meteva.base.grid_data(grid0)
    for i in range(len(level_list)):
        sta1 =  sta.loc[sta['level'] == level_list[i]]
        for j in range(len(times)):
            sta2 = sta1.loc[sta1['time'] == times[j]]
            for k in range(len(dtime_list)):
                sta3 = sta2.loc[sta2['dtime'] == dtime_list[k]]
                ig = ((sta3['lon'].values - slon) // dlon).astype(dtype='int16')
                jg = ((sta3['lat'].values - slat) // dlat).astype(dtype='int16')
                for m in range(len(member_list)):
                    dat = np.zeros((grid0.nlat, grid0.nlon))
                    dat[jg, ig] = sta3.loc[:, member_list[m]]
                    grd.values[m,i,j,k,:,:] = dat[:,:]
    return grd


def trans_contours_to_sta(m14,station,grade_list):

    contours = m14["closed_contours"]
    ncontour = len(contours["cn_label"])
    ploys_dict = {}

    for g in range(len(grade_list)):
        grade = grade_list[g]
        ploys_dict[g] = []
        for n in range(ncontour):
            if float(contours["cn_label"][n]) == grade or (int(float(contours["cn_label"][n])) ==0 and g ==0):
                line = contours["cn_xyz"][n][:,0:2]
                ploys_dict[g].append(line)

    sta_fo = station.copy()  #  meteva.base.interp_gs_nearest(grd_fo, sta_ob_in)

    nsta = len(sta_fo.index)

    grade_list1 = [0]
    grade_list1.extend(grade_list)
    for i in range(nsta):
        point1 = [sta_fo.iloc[i,4],sta_fo.iloc[i,5]]
        gg = 0
        for g in range(len(grade_list)):
            ploys = ploys_dict[g]
            inploy = meteva.base.tool.math_tools.isPoiWithinPoly(point1,ploys)
            if not inploy:
                break
            else:
                gg = g + 1
        sta_fo.iloc[i, -1] = grade_list1[gg]

    return sta_fo



def move_fo_time(data,dtime,keep_minus_dtime = True):
    if isinstance(data, pd.DataFrame):
        sta1 = data.copy()
        sta1["time"] = data["time"] + dtime* np.timedelta64(1, 'h')
        sta1["dtime"] = data["dtime"] - dtime
        if not keep_minus_dtime: sta1 = meteva.base.between_dtime_range(sta1,0,10000)
        return sta1
    else:
        grd1 = data.copy()
        grd1.coords["time"]= grd1.coords["time"].values[:] + dtime* np.timedelta64(1, 'h')

        grd1.coords["dtime"] =grd1.coords["dtime"].values[:] -  dtime
        if not keep_minus_dtime:grd1 = meteva.base.between_dtime_range(grd1,0,10000)
        return grd1

def tran_ut_nature_local_time(sta):
    '''
    :param sta: 世界时站点数据
    :return:  当地时站点数据
    '''
    sta_lt = copy.deepcopy(sta)
    lon = copy.deepcopy(sta_lt["lon"].values)
    lon[lon>180] -= 360
    hour = np.round((lon/15)).astype(np.int)
    sta_lt["time"] += hour * np.timedelta64(1, 'h')

    return sta_lt

def tran_ut_Administrative_local_time(sta,id_zone):
    '''
    :param sta: 世界时站点数据
    :return:  当地时站点数据
    '''

    sta_zone = meteva.base.sta_data(id_zone)
    sta_com = meteva.base.combine_expand_IV(sta,sta_zone)
    sta_com["time"] += sta_com.iloc[:,-1] * np.timedelta64(1, 'h')
    sta_com = sta_com.iloc[:,:-1]

    return sta_com


# -*-coding:utf-8-*-
import math
import numpy as np


def bearing(point1, point2, deg=True, aty="compass"):
    point1 = np.array(point1)
    point2 = np.array(point2)
    if deg:
        point1[:, 1] = point1[:, 1] * math.pi / 180
        point2[:, 1] = point2[:, 1] * math.pi / 180
    dlon = np.array(point1[:, 0] - point2[:, 0])
    if deg:
        dlon = dlon * math.pi / 180

    s = np.multiply(math.cos(point2[:, 1]), math.sin(dlon))
    c_val = np.multiply(math.cos(point1[:, 1]), math.sin(point2[:, 1])) - np.multiply(
        np.multiply(math.sin(point1[:, 1]), math.cos(point2[:, 1])), math.cos(dlon))
    out = np.array(math.atan2(s, c_val))
    if deg:
        out = out * 180 / math.pi
    if aty == "radial":
        out[(out >= 0) & (out <= 45)] = abs(out[(out >= 0) & (out <= 45)] - 45) + 45
        out[out > 45] = 90 - out[out > 45]
        out[out < 0] = out[out < 0] + 360
    return out



# -*-coding:utf-8-*-


def get_attributes_for_feat(feat):
    attribute = {'Type': feat['Type'], 'xrange': feat['xrange'], 'yrange': feat['yrange'],
                 'dim': feat['dim'], 'xstep': feat['xstep'], 'ystep': feat['ystep'], 'area': feat['area'],
                 'warnings': feat['warnings'], 'xcol': feat['xcol'], 'yrow': feat['yrow']}
    return attribute


def remove_key_from_list(origin_list, remove_list):
    for i, val in enumerate(remove_list):
        origin_list.remove(val)
    return origin_list



import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签
plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
from meteva.method.yes_or_no.score import *
import meteva


def performance(ob, fo,grade_list=[1e-30],compare =">=",compair = None, member_list=None,x_y = "sr_pod", save_path=None,show = False,dpi = 300, title="综合表现图",
                sup_fontsize =10,width = None,height = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    if compair is not None:
        print("warning: the argument compair will be abolished, please use compare instead\n警告：参数compair 将被废除，以后请使用参数compare代替")
        compare = compair
    if compare not in [">=",">","<","<="]:
        print("compare 参数只能是 >=   >  <  <=  中的一种")
        return

    hfmc_array = hfmc(ob, fo, grade_list,compare=compare)
    pod = pod_hfmc(hfmc_array)
    sr = sr_hfmc(hfmc_array)
    performance_sr_pod(sr,pod,grade_list, member_list,x_y = x_y, save_path=save_path,show = show,dpi = dpi, title=title,
                sup_fontsize =sup_fontsize,width = width,height = height)



def performance_mr_far(mr,far,grade_list, member_list,x_y = "sr_pod", save_path=None,show = False,dpi = 300, title="综合表现图",
                sup_fontsize =10,width = None,height = None,masker = None,masker_size = None):
    sr = 1 - far
    pod = 1 - mr
    performance_sr_pod(sr,pod,grade_list, member_list,x_y = x_y, save_path=save_path,show = show,dpi = dpi, title=title,
                sup_fontsize =sup_fontsize,width = width,height = height,masker = masker,masker_size = masker_size)

def performance_sr_pod(sr,pod,grade_list, member_list,x_y = "sr_pod", save_path=None,show = False,dpi = 300, title="综合表现图",
                sup_fontsize =10,width = None,height = None,masker = None,masker_size = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    leftw = 0.6
    rightw = 2
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3.7
    axis_size_y = 3.5
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    fig = plt.figure(figsize=(width, height),dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])

    x = np.arange(0.0001, 1, 0.0001)
    bias_list = [0.2, 0.4, 0.6, 0.8, 1, 1.25, 1.67, 2.5, 5]
    ts_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
    for i in range(len(bias_list)):
        bias = bias_list[i]
        y1 = bias * x
        x2 = x[y1 < 1]
        y2 = y1[y1 < 1]
        if bias < 1:
            # bias <1 的 线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.01, bias, "bias=" + str(bias),fontsize = sup_fontsize * 0.8)
        elif bias > 1:
            # bias》1的线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.0 / bias - 0.05, 1.02, "bias=" + str(bias),fontsize = sup_fontsize * 0.8)
        else:
            # bias ==1 的线
            ax1.plot(x2, y2, '-', color='k', linewidth=0.5)

    for i in range(len(ts_list)):
        ts = ts_list[i]
        hf = 1
        x2 = np.arange(ts, 1, 0.001)
        hit = hf * x2
        hfm = hit / ts
        m = hfm - hf
        y2 = hit / (hit + m)
        # ts 的线
        plt.plot(x2, y2, "--", color="y", linewidth=0.5)
        error = np.abs(y2 - x2)
        index = np.argmin(error)
        sx = x2[index] + 0.02
        sy = y2[index] - 0.02
        ax1.text(sx, sy, "ts=" + str(ts))


    new_sr = sr.reshape((-1, len(grade_list)))
    new_pod = pod.reshape((-1, len(grade_list)))

    new_sr_shape = new_sr.shape
    label = []
    legend_num = new_sr_shape[0]
    if member_list is None:
        if legend_num == 1:
            label.append('预报')
        else:
            for i in range(legend_num):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)

    ngrade = len(grade_list)
    colors = meteva.base.color_tools.get_color_list(legend_num)

    masker_list = []
    if masker is None:
        masker_list = ['o',  'v',  's','p',"P", "*", 'h',"X","d","1","+","x",".","^","<",">",
                  "2","3","4","8","H","D","|","_"]
    else:
        if isinstance(masker,list):
            masker_list = masker
        else:
            for i in range(ngrade):
                masker_list.append(masker)


    if masker_size is None:
        masker_size_list = np.ones(ngrade) * 6
    else:
        if isinstance(masker_size,list):
            masker_size_list = masker_size
        else:
            masker_size_list = np.ones(ngrade) * masker_size

    if isinstance(grade_list[0],str):
        grade_name_list = grade_list
    else:
        grade_name_list = ['grade:'+str(i)for i in grade_list]

    if masker_size_list is None:
        masker_size_list = np.ones(len(grade_list)).tolist()

    a_list= []
    grade_num = len(grade_list)
    if legend_num>1 and grade_num>1:
        for line in range(legend_num):
            for i in range(len(grade_list)):
                ax1.plot(new_sr[line, i], new_pod[line, i], masker_list[i],label = i*line, color=colors[line], markersize=masker_size_list[i])
                a_list.append(i*line)
        lines,label1 = ax1.get_legend_handles_labels()
        legend2 = ax1.legend(lines[0:len(lines):len(grade_list)],label,loc="upper right",
                             bbox_to_anchor=(1.5, 1),ncol=1, fontsize=sup_fontsize * 0.9)
        legend1=ax1.legend(lines[:len(grade_list)],grade_name_list,loc="lower right",
                           bbox_to_anchor=(1.5, 0), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)
        ax1.add_artist(legend2)
    elif legend_num>1:
        for line in range(legend_num):
            i = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], masker_list[0], label=i * line, color=colors[line], markersize=masker_size_list[0])
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend2 = ax1.legend(lines[0:len(lines):len(grade_list)], label, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend2)

    elif grade_num>1:
        colors = meteva.base.color_tools.get_color_list(grade_num)
        for i in range(grade_num):
            line = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], masker_list[i], label=i * line, color=colors[i], markersize=masker_size_list[i])
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend1=ax1.legend(lines[:len(grade_list)],grade_name_list,loc="upper right",
                           bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)

    ax1.set_xlim(0, 1)
    ax1.set_ylim(0, 1)
    if x_y == "sr_pod":
        ax1.set_xlabel("成功率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("命中率", fontsize=sup_fontsize * 0.9)
    else:
        ax1.set_xlabel("空报率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("漏报率", fontsize=sup_fontsize * 0.9)
        x = np.arange(0,1.01,0.2)
        ax1.set_xticks(x)
        ax1.set_xticklabels(np.round(1-x,1))
        y = np.arange(0,1.01,0.2)
        ax1.set_yticks(y)
        ax1.set_yticklabels(np.round(1-y,1))

    title = title + "\n"
    ax1.set_title(title,fontsize = sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()



def performance_hfmc(hfmc_array,axis_list_list,suplot_lengend = [1,0],save_dir = None):
    pass
    '''
    :param ob:
    :param fo:
    :param grade_list:
    :return:

    grade_count = len(grade_list)
    group_count = len(group_list)
    if grade_count == 1:
        #只有一种等级的
        pass
    elif group_list == 1:
        pass
    else:
        pass



    if grade_count == 1 or group_count == 1:
        column = 1
        row = 1
        if grade_count == 1:
            legend_list = group_list
        else:
            legend_list = grade_list
        subplot_count = 1
    else:
        column = 2
        row = int(math.ceil(grade_count/2))
        legend_list = group_list
        subplot_count = grade_count

    plt.figure(figsize=(column * 5, row * 4.5))
    plt.subplots_adjust(wspace = 0.5,hspace = 0.3)

    pod = pod_hfmc(hfmc_array)
    sr = sr_hfmc(hfmc_array)
    x = np.arange(0.0001, 1, 0.0001)
    bias_list = [0.2, 0.4, 0.6, 0.8, 1, 1.25, 1.67, 2.5, 5]
    ts_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]


    for s in range(subplot_count):
        plt.subplot(column, row, s+1)
        for i in range(len(bias_list)):
            bias = bias_list[i]
            y1 = bias * x
            x2 = x[y1 < 1]
            y2 = y1[y1 < 1]
            if bias < 1:
                plt.plot(x2, y2, '--', color='k', linewidth=0.5)
                plt.text(1.01, bias, "bias=" + str(bias))
            elif bias > 1:
                plt.plot(x2, y2, '--', color='k', linewidth=0.5)
                plt.text(1.0 / bias - 0.05, 1.02, "bias=" + str(bias))
            else:
                plt.plot(x2, y2, '-', color='k', linewidth=0.5)

        for i in range(len(ts_list)):
            ts = ts_list[i]
            hf = 1
            x2 = np.arange(ts, 1, 0.001)
            hit = hf * x2
            hfm = hit / ts
            m = hfm - hf
            y2 = hit / (hit + m)
            plt.plot(x2, y2, "--", color="y", linewidth=0.5)
            error = np.abs(y2 - x2)
            index = np.argmin(error)
            sx = x2[index] + 0.02
            sy = y2[index] - 0.02
            plt.text(sx, sy, "ts=" + str(ts))

        colors = cm.get_cmap('rainbow', 128)
        for i in range(sr.shape[0]):
            color_grade = (i +0.5) /len(legend_list)
            plt.plot(sr[i,s], pod[i,s], 'o', color=colors(color_grade),markersize=12,label = legend_list[i])

        plt.xlim(0, 1)
        plt.ylim(0, 1)
        plt.xlabel("成功率",fontsize = 14)
        plt.ylabel("命中率",fontsize = 14)


    if len(legend_list) < 10:
        legend_ncol = len(legend_list)
    else:
        legend_ncol = int(math.ceil(len(legend_list)/2))
    plt.legend(loc='upper center', bbox_to_anchor=(0, -0.05,1,1),ncol = legend_ncol,bbox_transform=plt.gcf().transFigure)
    if save_path is None:
        plt.show()
    else:
        plt.savefig(save_path)
    '''



from .feature_finder import feature_finder,feature_finder_and_merge
from .centmatch import centmatch
from .merge_force import merge_force
from .consistent import unimerge,unimatch
from .feature_match_analyzer import feature_merged_analyzer
import meteva
import json
import copy
from .plot import plot_value_and_label,plot_feature


def operate(grd_ob, grd_fo, smooth, threshold,minsize,compare = ">=",match_method = centmatch,near_dis =100,
            near_rate =0.3,cover_dis = 100,cover_rate =0.5,summary =True,
            save_dir = None,cmap = "rain_24h",clevs = None,show = False,dpi = 300):
    if match_method.__name__ == unimatch.__name__:
        if not isinstance(smooth,list):
            smooth = [smooth,smooth]
        if not isinstance(threshold,list):
            threshold = [threshold,threshold]
        if not isinstance(minsize,list):
            minsize = [minsize,minsize]
        if not isinstance(near_dis,list):
            near_dis = [near_dis,near_dis]
        if not isinstance(near_rate,list):
            near_rate = [near_rate,near_rate]

        look_ob = feature_finder_and_merge(grd_ob,smooth[0],threshold[0],minsize[0],compare = compare,near_dis=near_dis[0],
                                           near_rate = near_rate[0])
        look_fo = feature_finder_and_merge(grd_fo, smooth[1], threshold[1], minsize[1], compare=compare,
                                           near_dis=near_dis[1],
                                           near_rate=near_rate[1])
        matched_fo = unimatch(look_ob,look_fo,cover_dis,cover_rate)
        look_merge = unimerge(look_ob,matched_fo)
    else:
        look_ff = feature_finder(grd_ob,grd_fo,smooth=smooth,threshold=threshold,minsize=minsize,compare=compare)
        loof_match = match_method(look_ff)
        look_merge = merge_force(loof_match)
    features = feature_merged_analyzer(look_merge,summary=summary)
    time1 = meteva.base.all_type_time_to_datetime(grd_fo["time"].values[0])
    dtime1 = grd_fo["dtime"].values[0]


    if save_dir is not None:
        label_save_path = save_dir + "/label/YYYYMMDDHH.TTT.nc"
        label_save_path = meteva.base.get_path(label_save_path,time1,dtime1)
        meteva.base.creat_path(label_save_path)
        grid1 = meteva.base.get_grid_of_data(grd_fo)
        grid1.members = ["ob",grid1.members[0]]
        labels = meteva.base.grid_data(grid1)
        labels.values[0,...] = look_merge["grd_ob_label"].values[...]
        labels.values[1, ...] = look_merge["grd_fo_label"].values[...]
        meteva.base.write_griddata_to_nc(labels,save_path=label_save_path)
        print("目标编号场已经输出至 "+label_save_path)


        png_save_path = save_dir +"/png/YYYYMMDDHH.TTT.png"
        png_save_path =  meteva.base.get_path(png_save_path,time1,dtime1)
        meteva.base.creat_path(png_save_path)
        plot_value_and_label(look_merge,save_path=png_save_path,show=show,cmap = cmap,clevs = clevs,dpi=dpi)


        json_save_path = save_dir +"/json/YYYYMMDDHH.TTT.txt"
        json_save_path =  meteva.base.get_path(json_save_path,time1,dtime1)
        meteva.base.creat_path(json_save_path)
        nmatch = features["match_count"]
        #json_str = json.dumps(features)
        #print(json_str)

        features_list = copy.deepcopy(features)
        if  features["interester"] is not None:
            features_list["interester"] = features["interester"].tolist()
        label_list_matched = look_merge["label_list_matched"]
        for i in label_list_matched:
            features_list[i]["feature_props"]["ob"]["intensity"] =features[i]["feature_props"]["ob"]["intensity"][4].tolist()
            features_list[i]["feature_props"]["fo"]["intensity"] = features[i]["feature_props"]["fo"]["intensity"][4].tolist()
        for i in label_list_matched:
            q = [0,5,10,25,50, 75, 90, 95, 100]
            for k in range(len(q)):
                features_list[i]["feature_props"]["ob"]["intensity_"+str(q[k])] =features[i]["feature_props"]["ob"]["intensity"][k].tolist()
                features_list[i]["feature_props"]["fo"]["intensity_"+str(q[k])] = features[i]["feature_props"]["fo"]["intensity"][k].tolist()

        miss_labels = look_merge["unmatched"]['ob']
        for i in miss_labels:
            features_list[i]["feature_props"]["ob"]["intensity"] =features[i]["feature_props"]["ob"]["intensity"][4].tolist()

        for i in miss_labels:
            q = [0,5,10,25,50, 75, 90, 95, 100]
            for k in range(len(q)):
                features_list[i]["feature_props"]["ob"]["intensity_"+str(q[k])] =features[i]["feature_props"]["ob"]["intensity"][k].tolist()

        false_alarm_labels = look_merge["unmatched"]['fo']
        for i in false_alarm_labels:
            features_list[i]["feature_props"]["fo"]["intensity"] =features[i]["feature_props"]["fo"]["intensity"][4].tolist()

        for i in false_alarm_labels:
            q = [0,5,10,25,50, 75, 90, 95, 100]
            for k in range(len(q)):
                features_list[i]["feature_props"]["fo"]["intensity_"+str(q[k])] =features[i]["feature_props"]["fo"]["intensity"][k].tolist()

        json_str = json.dumps(features_list)

        br = open(json_save_path, 'w')
        br.write(json_str)
        br.close()
        print("目标属性检验结果已经输出至 "+json_save_path)

        png_save_path = save_dir + "/table/YYYYMMDDHH.TTT.png"
        png_save_path = meteva.base.get_path(png_save_path, time1, dtime1)
        meteva.base.creat_path(png_save_path)
        plot_feature(features, save_path=png_save_path, show=show,dpi=dpi)

    return features





import datetime
import meteva
import os

from multiprocessing import Process,cpu_count

para_example= {
    "cpu_count":1,
    "ip_port_file":r"H:\test_data\ip_port.txt",
    "local_binary_dir": r"O:\data\mdfs",
    "local_sta_dir": r"O:\data\sta",
    "local_grid_dir":r"O:\data\grid",
    "max_save_day": 14,
    "sta_origin_dirs": [
        ["mdfs:///SURFACE/QC_BY_FSOL/RAIN01_ALL_STATION/", [[0, 2359]]],
        ["mdfs:///SURFACE/QC_BY_FSOL/WIND_AVERAGE_2MIN_ALL_STATION/", [[0, 2359]]],
        ["mdfs:///SURFACE/QC_BY_FSOL/TMP_ALL_STATION/", [[0, 2359]]],
    ],
    "grid_origin_dirs": {
        "NWFD_SCMOC": [
            ["NWFD_SCMOC/RAIN03", [[1100, 1900],[2100,2300]]],
            ["NWFD_SCMOC/TMP/2M_ABOVE_GROUND", [[1100, 1200],[2100,2359]]],
            ["NWFD_SCMOC/WIND/10M_ABOVE_GROUND", [[1100, 1200],[2100,2359]]],
        ],
        "EWMWF": [
            ["ECMWF_HR/TMP_2M/", [[1100, 1200],[2200,2359]]],
            ["ECMWF_HR/APCP/", [[1100, 1200],[2200,2359]]],
            ["mdfs:///ECMWF_HR/WIND_10M/", [[1100, 1200],[2200,2359]]],
        ],
        "GRAPES_GFS": [
            ["mdfs:///GRAPES_GFS/WIND/10M_ABOVE_GROUND/", [[1100, 1200],[2200,2359]]],
            ["GRAPES_GFS/TMP/2M_ABOVE_GROUND", [[1100, 1200],[2200,2359]]],
            ["GRAPES_GFS/APCP", [[1100, 1200],[2200,2359]]],
        ],
    }
}


def download_one_cpu(k,ip,port,local_binary_dir,local_sta_dir,local_grid_dir,file_sta_list,file_grid_list):
    service = meteva.base.io.GDSDataService(ip, port)
    for filepath in file_sta_list:
        dir,file = os.path.split(filepath)
        save_path = local_binary_dir + "/" + dir + "/" + file
        dati_str = meteva.product.application.get_dati_str_of_path(save_path)
        try:
            status, response = service.getData(dir, file)
            ByteArrayResult = meteva.base.io.DataBlock_pb2.ByteArrayResult()
            if status == 200:
                ByteArrayResult.ParseFromString(response)
                if ByteArrayResult is not None:
                    byteArray = ByteArrayResult.byteArray
                    save_path_sta = local_sta_dir + "/" + dir + "/" + dati_str + "/" + file
                    meteva.base.tool.path_tools.creat_path(save_path_sta)
                    br = open(save_path_sta, 'wb')
                    br.write(byteArray)
                    br.close()
                    print(save_path_sta)
                    print(k)
        except Exception as e:
            print(e)

    for filepath in file_grid_list:
        dir,file = os.path.split(filepath)
        save_path = local_binary_dir + "/" + dir + "/" + file
        dati_str = meteva.product.application.get_dati_str_of_path(save_path)
        try:
            status, response = service.getData(dir, file)
            ByteArrayResult = meteva.base.io.DataBlock_pb2.ByteArrayResult()
            if status == 200:
                ByteArrayResult.ParseFromString(response)
                if ByteArrayResult is not None:
                    byteArray = ByteArrayResult.byteArray
                    meteva.base.tool.path_tools.creat_path(save_path)
                    br = open(save_path, 'wb')
                    br.write(byteArray)
                    br.close()
                    if dir.upper().find("WIND") >= 0:
                        if (dir.upper().find("GUST")) >= 0:
                            grd = meteva.base.io.decode_griddata_from_gds_byteArray(byteArray)
                        else:
                            grd = meteva.base.io.decode_gridwind_from_gds_byteArray(byteArray)
                    else:
                        grd = meteva.base.io.decode_griddata_from_gds_byteArray(byteArray)
                    save_path_nc = local_grid_dir + "/" + dir + "/" + dati_str + "/" + file + ".nc"
                    meteva.base.write_griddata_to_nc(grd, save_path_nc, creat_dir=True,show=True)
                    print(k)
        except Exception as e:
            print(e)

def download_mp(ip,port,local_binary_dir,local_sta_dir,local_grid_dir,download_sta_list,download_grid_list,multi_pro_num):
    max_pro_num = cpu_count() - 2
    if multi_pro_num > max_pro_num:
        multi_pro_num = max_pro_num

    file_sta_dict_list = {}
    file_grid_dict_list = {}
    for i in range(multi_pro_num):
        file_sta_dict_list[i] = []
        file_grid_dict_list[i] = []

    for i in range(len(download_sta_list)):
        k = i % multi_pro_num
        file_sta_dict_list[k].append(download_sta_list[i])
    for i in range(len(download_grid_list)):
        k = i % multi_pro_num
        file_grid_dict_list[k].append(download_grid_list[i])

    PP = []
    for k in range(multi_pro_num):
        kwargs = {
            "k":k,
            "ip":ip,
            "port":port,
            "local_binary_dir": local_binary_dir,
            "local_sta_dir":local_sta_dir,
            "local_grid_dir":local_grid_dir,
            "file_sta_list":file_sta_dict_list[k],
            "file_grid_list":file_grid_dict_list[k]
        }
        #tmpp = Process(target=download_one_cup, args=(k,ip,port,local_binary_dir,local_sta_dir,local_grid_dir,file_sta_dict_list[k],file_grid_dict_list[k]))
        tmpp = Process(target=download_one_cpu,kwargs=kwargs)
        PP.append(tmpp)
    print('Waiting for all subprocesses done...')
    for pc in PP:
        pc.start()

    for pp in PP:
        pp.join()
    print('All subprocesses done.')


def in_op_time(hm,op_list_list):
    in_op = False
    #print(op_list_list)
    for down_set in op_list_list:
        if hm >= down_set[0] and hm <= down_set[1]:
            in_op = True
    return in_op

def download_from_gds(para):
    #遍历下载
    now = datetime.datetime.now()
    print("于"+str(now)+"开始下载")
    weekago = now - datetime.timedelta(days=7)
    hm = now.hour * 100 + now.minute
    ip,port = meteva.base.io.read_gds_ip_port(para["ip_port_file"])
    service = meteva.base.io.GDSDataService(ip, port)
    meteva.base.gds_ip_port = ip,port
    if (service is None):
        print("service is None")
        return
    download_sta_list = []
    for down_set in para["sta_origin_dirs"]:
        if in_op_time(hm,down_set[1]):
            dir_list = []
            meteva.base.tool.path_tools.get_gds_all_dir(down_set[0],dir_list,service=service)
            for dir in dir_list:
                file_list = meteva.base.tool.path_tools.get_gds_file_list_in_one_dir(dir)
                file_list.sort(reverse = True)
                for file in file_list:
                    dati = meteva.product.application.get_dati_of_path(file)
                    if dati < weekago:break
                    dati_str = meteva.product.application.get_dati_str_of_path(file)
                    save_path_sta = para["local_sta_dir"] + "/" + dir + "/" + dati_str + "/" + file
                    if not os.path.exists(save_path_sta):
                        download_sta_list.append(dir + "/" +file)

    download_grid_list = []
    for key in para["grid_origin_dirs"].keys():
        down_set_group  = para["grid_origin_dirs"][key]
        #print(down_set_group)
        for down_set in down_set_group:
            if in_op_time(hm, down_set[1]):
                dir_list = []
                meteva.base.tool.path_tools.get_gds_all_dir(down_set[0],dir_list,service=service)
                for dir in dir_list:
                    file_list = meteva.base.tool.path_tools.get_gds_file_list_in_one_dir(dir)
                    file_list.sort(reverse=True)
                    for file in file_list:
                        dati = meteva.product.application.get_dati_of_path(file)
                        if dati < weekago: break
                        save_path = para["local_binary_dir"] + "/" +dir+"/" +file
                        if not os.path.exists(save_path):
                            download_grid_list.append(dir + "/" + file)
                        else:
                            try:
                                dati_str = meteva.product.application.get_dati_str_of_path(save_path)
                                save_path_nc = para["local_grid_dir"] + "/" + dir + "/" + dati_str + "/" + file+".nc"
                                if not os.path.exists(save_path_nc):
                                    if dir.upper().find("WIND")>=0:
                                        if (dir.upper().find("GUST"))>=0:
                                            grd = meteva.base.read_griddata_from_gds_file(save_path)
                                        else:
                                            grd = meteva.base.read_gridwind_from_gds_file(save_path)
                                    else:
                                        grd = meteva.base.read_griddata_from_gds_file(save_path)
                                    if grd is not None:
                                        meteva.base.write_griddata_to_nc(grd, save_path_nc, creat_dir=True)
                                    else:
                                        print(save_path + "文件格式错误")
                            except Exception as e:
                                print(e)

    if len(download_sta_list) == 0 and len(download_grid_list) == 0:
        return
    else:
        download_mp(ip, port, para["local_binary_dir"], para["local_sta_dir"], para["local_grid_dir"], download_sta_list, download_grid_list,
                para["cpu_count"])
        download_from_gds(para)


def remove(dir,save_day):
    file_list = meteva.base.tool.path_tools.get_path_list_in_dir(dir)
    now = datetime.datetime.now()
    for file in file_list:
        dati = meteva.product.application.get_dati_of_path(file)
        dday = (now - dati).total_seconds()/(3600*24)
        if dday > save_day:
            os.remove(file)
    print(dir +"目录下"+str(save_day)+"天之前的数据已删除")



if __name__ == '__main__':
    pass
    download_from_gds(para_example)



